/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/


/***********************************************************************
MultiCam sample program - grablink_snapshot_trigger

This application demonstrates the Snapshot Acquisition Mode on a 
GRABLINK Board. 
The camera used is a ADIMEC 1000M.  By default, this       
program requires an area-scan camera connected on connector A.

When it is launched from command line, it waits for either a hardware 
trigger pulse on TRIG line or by pressing <t> key to generates a soft 
trigger which starts a frame acquisition. Press "ESC" key to close the 
application.

The program uses automatic memory allocation and works with both 32 and 
64 bit systems.
***********************************************************************/


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>

#include "/usr/local/euresys/multicam/include/multicam.h"

#ifndef BYTE
#define BYTE unsigned char
#endif

SDL_Surface* screen = NULL;
SDL_Overlay *overlay = NULL;

BOOL error; // Flagged on acquisition error

int sizeX;  // Width of the acquired image
int pitch;  // Pitch of the acquired image
int sizeY;  // Height of the acquired image

MCHANDLE hChannel; // Handle to the active channel


void SoftTrig()
{
	MCSTATUS status = 0;

	// Generate a soft trigger event (STRG)
	status = McSetParamInt(hChannel, MC_ForceTrig, MC_ForceTrig_TRIG);
	if (status != MC_OK) goto Finalize;

	printf("Soft trig\n");
	return ;

Finalize:
	if (status!=MC_OK)
	{
		printf("Acquire Error %d\n",status);
	}
	error = TRUE;


}

// check if <esc> was hit to exit application
BOOL CheckForQuit()
{
	SDL_Event event;

	// Poll for events, and handle the ones we care about.
	while (SDL_PollEvent(&event)) {
		switch (event.type) {
		case SDL_KEYUP:
			// If escape is pressed, return (and thus, quit)
			if (event.key.keysym.sym == SDLK_ESCAPE)
				return TRUE;
			if (event.key.keysym.sym == 0x74) 	// <t> key
				SoftTrig();							// emulate trigger
			break;
		case SDL_QUIT:
			return TRUE;
		}
	}

	return FALSE;
}

// Draw acquired image to screen using YUV overlay
void UpdateSurface(BYTE *pImage)
{
	SDL_Rect rect;
	uint8_t *overlayData;
	SDL_LockYUVOverlay(overlay);

	// Y plane	
	overlayData = overlay->pixels[0];
	memcpy(overlayData,pImage, sizeY*pitch);
	// U plane	
	overlayData = overlay->pixels[1];
	memset(overlayData,128, sizeX* sizeY /4);
	// V plane	
	overlayData = overlay->pixels[2];
	memset(overlayData,128, sizeX* sizeY /4);

	SDL_UnlockYUVOverlay(overlay);

	rect.x = 0;
	rect.y = 0;
	rect.w = sizeX;
	rect.h = sizeY;
	SDL_DisplayYUVOverlay(overlay, &rect);

}

//  Callback function registered to our channel. 
//    Called at the asynchronous signals.
void McCallback(PMCCALLBACKINFO CallBackInfo)
{
	MCHANDLE hSurface;
	BYTE *pImage;

	switch(CallBackInfo->Signal)
	{
	case MC_SIG_SURFACE_PROCESSING:
		hSurface = (MCHANDLE) CallBackInfo->SignalInfo;
		McGetParamPtr(hSurface,MC_SurfaceAddr,(PVOID*)&pImage);
		UpdateSurface(pImage);
		printf("Received new image\n");
		break;
	case MC_SIG_ACQUISITION_FAILURE:
		fprintf(stderr, "Acquisition Failure. Is a video signal connected ?\n");
		error = TRUE;
		break;
	default:
		fprintf(stderr,"Signal not handled:%d", CallBackInfo->Signal);
		error = TRUE;
		break;
	}
}


//  Create a channel and set its parameters
int InitializeMultiCam()
{
	MCSTATUS status = 0;


	int i;

	sizeX=0;
	sizeY=0;


	// In order to support a 10-tap camera on Grablink Full
	// BoardTopology need to be set to MC_BoardTopology_MONO_DECA
	// In order to support 2 cameras on Grablink Expert 2
	// BoardTopology need to be set to MC_BoardTopology_DUO
	// In all other case the default value will work properly 
	// and the parameter don't need to be set

	// Set the board topology to support 2 cameras (only with a Grablink Expert 2)
	// status = McSetParamInt(MC_BOARD + 0, MC_BoardTopology, MC_BoardTopology_DUO);
	// if (status != MC_OK) goto Finalize;

	// Set the board topology to support 10 taps mode (only with a Grablink Full)
	// status = McSetParamInt(MC_BOARD + 0, MC_BoardTopology, MC_BoardTopology_MONO_DECA);
	// if (status != MC_OK) goto Finalize;

	//  Create a channel.
	status = McCreate(MC_CHANNEL, &hChannel);
	if (status != MC_OK) goto Finalize;

	// Link the channel to a board. Here we take the first board.
	status = McSetParamInt(hChannel, MC_DriverIndex, 0);
	if (status != MC_OK) goto Finalize;

	// In order to use single camera on connector A
	// MC_Connector need to be set to A for Grablink Expert 2 and Grablink DualBase
	// For all the other Grablink boards the parameter has to be set to M  

	// For all GrabLink boards but Grablink Expert 2 and Dualbase
	status = McSetParamStr(hChannel, MC_Connector, "M");
	if (status != MC_OK) goto Finalize;
	// For Grablink Expert 2 and Dualbase
	// status = McSetParamStr(hChannel, MC_Connector, "A");
	// if (status != MC_OK) goto Finalize;

	// Choose the camera camfile
	status = McSetParamStr(hChannel, MC_CamFile, "/usr/local/euresys/multicam/cameras/BuiltIn/ADIMEC/1000m_P50RG");
	if (status != MC_OK) goto Finalize;

	// Choose the camera expose duration
	status = McSetParamInt(hChannel, MC_Expose_us, 33333);
	if (status != MC_OK) goto Finalize;

	// Configure triggering mode
	status = McSetParamInt(hChannel, MC_TrigMode, MC_TrigMode_COMBINED);
	if (status != MC_OK) goto Finalize;
	status = McSetParamInt(hChannel, MC_NextTrigMode, MC_NextTrigMode_SAME);
	if (status != MC_OK) goto Finalize;

	// Configure triggering line
	// A rising edge on the triggering line generates a trigger.
	// See the TrigLine Parameter and the board documentation for more details.
	status = McSetParamInt(hChannel, MC_TrigLine, MC_TrigLine_NOM);
	if (status != MC_OK) goto Finalize;
	status = McSetParamInt(hChannel, MC_TrigEdge, MC_TrigEdge_GOHIGH);
	if (status != MC_OK) goto Finalize;
	status = McSetParamInt(hChannel, MC_TrigFilter, MC_TrigFilter_ON);
	if (status != MC_OK) goto Finalize;
	
	// Parameter valid for all Grablink but Full, DualBase, Base
	// status = McSetParamInt(hChannel, MC_TrigCtl, MC_TrigCtl_ITTL);
	// Parameter valid only for Grablink Full, DualBase, Base
	status = McSetParamInt(hChannel, MC_TrigCtl, MC_TrigCtl_ISO);
	if (status != MC_OK) goto Finalize;

	// Retrieve channel size information.
	status= McGetParamInt(hChannel, MC_ImageSizeX, &sizeX);
	if (status != MC_OK) goto Finalize;
	status= McGetParamInt(hChannel, MC_ImageSizeY, &sizeY);
	if (status != MC_OK) goto Finalize;  
	status= McGetParamInt(hChannel, MC_BufferPitch, &pitch);
	if (status != MC_OK) goto Finalize; 

	// The number of images to acquire. One image per trigger
	status = McSetParamInt (hChannel, MC_SeqLength_Fr, MC_INDETERMINATE);
	if (status != MC_OK) goto Finalize;

	// Register our Callback function for the MultiCam asynchronous signals.
	status = McRegisterCallback(hChannel, McCallback, NULL);
	if (status != MC_OK) goto Finalize;

	// Enable the signals we need: 
	// MC_SIG_SURFACE_PROCESSING: acquisition done and locked for processing
	// MC_SIG_ACQUISITION_FAILURE: acquisition failed.
	status = McSetParamInt(hChannel, MC_SignalEnable + MC_SIG_SURFACE_PROCESSING, MC_SignalEnable_ON);
	if (status != MC_OK) goto Finalize;

	status = McSetParamInt(hChannel, MC_SignalEnable + MC_SIG_ACQUISITION_FAILURE, MC_SignalEnable_ON);
	if (status != MC_OK) goto Finalize;

	return 0;

Finalize:
	if (status!=MC_OK)
	{
		printf("InitChannel Error %d\n",status);
	}

	if (hChannel)
	{
		McDelete(hChannel);
	}

	return -1;

}

// Start the image acquisition, images are processed in the callback
void AcquireImages() 
{
	MCSTATUS status = 0;
	// Start Acquisitions for this channel. 
	status = McSetParamInt(hChannel, MC_ChannelState, MC_ChannelState_ACTIVE);
	if (status != MC_OK) goto Finalize;

	// wait until user press <esc> key
	fprintf(stderr,"Waiting for trigger\n");
	while (!CheckForQuit() && !error ) 
	{ 
		sleep(1);
	}

	status= McSetParamInt(hChannel, MC_ChannelState, MC_ChannelState_IDLE);

Finalize:
	if (status!=MC_OK)
	{
		printf("Acquire Error %d\n",status);
	}

	if (hChannel)
	{
		McDelete(hChannel);
	}
}

// Application entry point
int main(int argc, char* argv[])
{
	MCSTATUS status;
	hChannel = 0;
	error = FALSE;

	// Initialize the MultiCam driver. 
	status = McOpenDriver(NULL);
	if(status != MC_OK)
	{
		fprintf(stderr,"Error - Fail to initialize MultiCam\n");
		return -1;
	}

	// Activate an error log file
	status = McSetParamStr (MC_CONFIGURATION, MC_ErrorLog, "error.log");
	if(status != MC_OK)
	{
		fprintf(stderr,"Error - configuration log \n");
		return -1;
	}    
	// Initialize Multical channel
	if (InitializeMultiCam())
		return -1;

	// Initialize SDL library and create window
	SDL_Init( SDL_INIT_VIDEO );
	screen = SDL_SetVideoMode(sizeX, sizeY, 32, SDL_SWSURFACE | SDL_DOUBLEBUF);
	// Created overlay
	overlay = SDL_CreateYUVOverlay(pitch, sizeY, SDL_IYUV_OVERLAY, screen);

	// Acquire images
	fprintf(stderr,"Press <Esc> key to exit\n");
	fprintf(stderr,"Press <t> key to emulate trigger\n");
	AcquireImages();

	// Delete overlay
	SDL_FreeYUVOverlay(overlay);
	// Quit SDL
	SDL_Quit();

	// Close the MultiCam driver.
	McCloseDriver();

	return 0;
}

